package com.example.e13roomshoppinglist

import android.annotation.SuppressLint
import android.os.Bundle
import android.widget.Toast
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.viewModels
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.room.Room
import com.example.e13roomshoppinglist.ui.theme.E13RoomShoppingListTheme
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch

class MainActivity : ComponentActivity() {
    private val db by lazy {
        Room.databaseBuilder(
            applicationContext,
            ShoppingListDatabase::class.java,
            "shoppingListDB.db"
        ).build()
    }
    private val viewModel by viewModels<ShoppingListViewModel> {
        object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return ShoppingListViewModel(db.dao) as T
            }
        }
    }

    @OptIn(DelicateCoroutinesApi::class)
    @SuppressLint("CoroutineCreationDuringComposition")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContent {
            E13RoomShoppingListTheme {
                val state by viewModel.state.collectAsState()
                val ctx = LocalContext.current
                var toast by remember {
                    mutableStateOf<Toast?>(null)
                }
                var isDataLoadMsgShown by rememberSaveable {
                    mutableStateOf(false)
                }

                val showToast = { msg: String ->
                    toast?.cancel()
                    toast = Toast.makeText(ctx, msg, Toast.LENGTH_SHORT)
                    toast?.show()
                }

                Surface(
                    modifier = Modifier.fillMaxSize(),
                    color = MaterialTheme.colorScheme.background
                ) {
                    ShoppingListScreen(
                        state = state,
                        onEvent = viewModel::onEvent,
                        onShowToast = showToast
                    )
                    if (!isDataLoadMsgShown) {
                        GlobalScope.launch {
                            delay(2000)
                            runOnUiThread {
                                val itemCount = state.shoppingListItems.count()
                                if (itemCount == 0) {
                                    showToast("Database is empty!")
                                } else {
                                    showToast("Loaded $itemCount item${if (itemCount==1) "" else "s"} from database!")
                                }
                                isDataLoadMsgShown = true
                            }
                        }
                    }
                }
            }
        }
    }
}
